#----------------------------------------------------------------------
# $Id: portforwarding.pm,v 1.31 2003/04/08 15:28:55 msoulier Exp $
# vim: ft=perl ts=4 sw=4 et:
#----------------------------------------------------------------------
# copyright (C) 2002 Mitel Networks Corporation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Mitel Networks
# Please visit our web site www.e-smith.com for details.
#----------------------------------------------------------------------

package esmith::FormMagick::Panel::portforwarding;

use strict;
use esmith::ConfigDB;
use esmith::FormMagick;
use esmith::util;
use esmith::util::network qw(isValidIP);
use esmith::cgi;
use Exporter;

use constant TRUE => 1;
use constant FALSE => 0;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw(
    show_port_forwards create_new validate_source_port
    validate_destination_port display_create_summary
    );

our $VERSION = sprintf '%d.%03d', q$Revision: 1.31 $ =~ /: (\d+).(\d+)/;
our $db = esmith::ConfigDB->open
    || die "Can't open configuration database: $!\n";

=head1 NAME

esmith::FormMagick::Panels::portforwarding - useful panel functions

=head1 SYNOPSIS

    use esmith::FormMagick::Panels::portforwarding

    my $panel = esmith::FormMagick::Panel::portforwarding->new();
    $panel->display();

=head1 DESCRIPTION

This module is the backend to the portforwarding panel, responsible for
supplying all functions used by that panel. It is a subclass of
esmith::FormMagick itself, so it inherits the functionality of a FormMagick
object.

=head2 new

This is the class constructor.

=begin testing

$ENV{ESMITH_CONFIG_DB} = "10e-smith-base/configuration.conf";

use_ok('esmith::FormMagick::Panels::portforwarding');
our $panel;
ok($panel = esmith::FormMagick::Panels::portforwarding->new(),
   "Create panel object");
isa_ok($panel, 'esmith::FormMagick::Panels::portforwarding');

=end testing

=cut

sub new {
    my $class = ref($_[0]) || $_[0];
    my $self = esmith::FormMagick->new();
    bless $self, $class;
    # Uncomment the following line for debugging.
    #$self->debug(TRUE);
    return $self;
}

=head2 show_port_forwards

This method displays the data on currently forwarded ports on
the system.

=cut

sub show_port_forwards {
    my $self = shift;
    my $q = $self->cgi;

    my $masq = $db->get('masq')
        || ($self->error('ERR_NO_MASQ_RECORD') and return undef);
    my $empty = 0;
    my %tcpforwards = split /,/, $masq->prop('TCPForwards');
    my %udpforwards = split /,/, $masq->prop('UDPForwards');
    $empty = 1 if not %tcpforwards and not %udpforwards;

    my %forwards = ();
    $forwards{TCP} = \%tcpforwards;
    $forwards{UDP} = \%udpforwards;

    unless ($empty) {
        print $q->Tr(
            $q->td({-colspan => 2}, 
                $q->p($self->localise('SHOW_FORWARDS')))),"\n";

        my $q = $self->{cgi};
        print "<tr><td colspan=\"2\">";
        print $q->start_table({-class => 'sme-border'}), "\n        ";
        print $q->Tr(
            esmith::cgi::genSmallCell(
                $q,
                $self->localise('PROTOCOL'),
                "header"
            ), "        ",
            esmith::cgi::genSmallCell(
                $q,
                $self->localise('SOURCE_PORT'),
                "header"
            ), "        ",
            esmith::cgi::genSmallCell(
                $q,
                $self->localise('DESTINATION_HOST'),
                "header"
            ), "        ",
            esmith::cgi::genSmallCell(
                $q,
                $self->localise('DESTINATION_PORT'),
                "header",
            ), "        ",
            esmith::cgi::genSmallCell(
                $q,
                $self->localise('ACTION'),
                "header",
            ), "\n        ",
        );
        foreach my $proto (sort keys %forwards) {
            if (%{ $forwards{$proto} }) {
                foreach my $sport (keys %{ $forwards{$proto} }) {
                    my ($dhost, $dport) = split /:/, 
                                          $forwards{$proto}->{$sport};
                    print $q->Tr(
                        esmith::cgi::genSmallCell($q, $proto),
                            "        ",
                        esmith::cgi::genSmallCell($q, $sport),
                            "        ",
                        esmith::cgi::genSmallCell($q, $dhost),
                            "        ",
                        esmith::cgi::genSmallCell($q, $dport || '&nbsp'),
                            "        ",
                        esmith::cgi::genSmallCell(
                            $q,
                            $q->a({href => $q->url(-absolute => 1)
                                    . "?page=3&Next=Next&protocol=$proto&"
                                    . "source_port=$sport&"
                                    . "destination_host=$dhost&"
                                    . "destination_port=$dport"},
                                $self->localise("REMOVE"))
                        ),
                            "\n        ",
                    );
                }
            }
        }
        print $q->end_table,"\n";
        print '</td></tr>';

    }
    else {
        print '<tr><td colspan="2">' .
            $self->localise('NO_FORWARDS') .
            '</td></tr>';
    }
    return undef;
}

=head2 validate_source_port

This method validates the source port field in the new port forward page.

=cut

sub validate_source_port {
    my $self = shift;
    my $sport = $self->{cgi}->param('source_port');
    $sport =~ s/^\s+|\s+$//g;
    # If this is a port range, split it up and validate it individually.
    my @ports = ();
    if ($sport =~ /-/)
    {
        @ports = split /-/, $sport;
        if (@ports > 2)
        {
            $self->debug_msg("found more than 2 ports: @ports");
            return $self->localise('ERR_BADPORT');
        }
    }
    else
    {
        push @ports, $sport;
    }
    $self->debug_msg("the ports array is: @ports");
    foreach my $port (@ports)
    {
        $self->debug_msg("looping on port $port");
        if (! $self->isValidPort($port))
        {
            $self->debug_msg("returning: " . $self->localise('ERR_BADPORT'));
            return $self->localise('ERR_BADPORT');
        }
    }
    return 'OK';
}

=head2 validate_destination_port

This method validates the destination port field in the new port
forward page.

=cut

sub validate_destination_port {
    my $self = shift;
    my $dport = $self->{cgi}->param('destination_port');
    $dport =~ s/^\s+|\s+$//g;
    # If the dport is empty, that's ok. 
    return 'OK' if not $dport;

    # If this is a port range, split it up and validate it individually.
    my @ports = ();
    if ($dport =~ /-/)
    {
        @ports = split /-/, $dport;
        if (@ports > 2)
        {
            $self->debug_msg("found more than 2 ports: @ports");
            return $self->localise('ERR_BADPORT');
        }
    }
    else
    {
        push @ports, $dport;
    }
    $self->debug_msg("the ports array is: @ports");

    foreach my $port (@ports)
    {
        $self->debug_msg("looping on port $port");
        if (! $self->isValidPort($port))
        {
            $self->debug_msg("returning: " . $self->localise('ERR_BADPORT'));
            return $self->localise('ERR_BADPORT');
        }
    }
    return 'OK';
}

=head2 isValidPort

Test for a valid port.
FIXME: Remove this when 5.6 is no longer supported, and use
esmith::util::network::isValidPort instead.

=begin testing

@badports = (98765434, -183, 0, 'bad port', 'a');
@goodports = (67, 23, 1, 54736);

foreach $port (@badports) {
    $panel->{cgi}->param('destination_port' => $port);
    isnt($panel->validate_source_port(), "OK");
}
foreach $port (@goodports) {
    $panel->{cgi}->param('source_port' => $port);
    is($panel->validate_source_port(), "OK");
}

=end testing

=cut

sub isValidPort() {
    my $self = shift;
    my $port = shift;

    return FALSE unless defined $port;

    if (($port =~ /^\d+$/) &&
        ($port > 0) &&
        ($port < 65536))
    {
        return TRUE;
    }
    else {
        return FALSE;
    }
}

=head2 validate_destination_host

The purpose of this method is to validate the destination host field in the
new port forward page.

=cut

sub validate_destination_host {
    my $self = shift;
    my $dhost = $self->{cgi}->param('destination_host');
    $dhost =~ s/^\s+|\s+$//g;
    if (isValidIP($dhost)) {
        return 'OK';
    }
    else {
        return $self->localise('ERR_BADIP');
    }
}

=head2 display_summary_create

This is a wrapper for the display_summary method, to call it in create mode. 

=cut

sub display_summary_create {
    my $self = shift;
    $self->display_summary('create');
}

=head2 display_summary_remove

This is a wrapper for the display_summary method, to call it in remove mode.

=cut

sub display_summary_remove {
    my $self = shift;
    $self->display_summary('remove');
}

=head2 display_create_summary

This method's purpose is to display a summary of the rule about to be added.

=cut

sub display_summary {
    my $self = shift;
    my $mode = shift;
    my $save = $self->localise('SAVE');
    my $cancel = $self->localise('CANCEL');
    my $output = "";
    my $q = $self->{cgi};
    $self->debug_msg("start of method");

    print "<tr><td colspan=\"2\">";

    my $description = "";
    if ($mode eq 'create') {
        $description = $self->localise('SUMMARY_ADD_DESC');
    }
    elsif ($mode eq 'remove') {
        $description = $self->localise('SUMMARY_REMOVE_DESC');
    }
    else {
        $self->error('ERR_UNSUPPORTED_MODE');
        return undef;
    }

    print $q->p($description);

    print $q->start_table({-class => 'sme-border'});
    foreach my $tablearrayref (
            [$self->localise('PROTOCOL')
                => $q->param('protocol')],
            [$self->localise('SOURCE_PORT')
                => $q->param('source_port')],
            [$self->localise('DESTINATION_PORT')
                => $q->param('destination_port') || '&nbsp;'],
            [$self->localise('DESTINATION_HOST')
                => $q->param('destination_host')],
        )
    {
        print $q->Tr($q->td({-class => 'sme-border-right'}, $tablearrayref->[0]),
                     esmith::cgi::genSmallCell($q, $tablearrayref->[1]),
                     "\n        "
        );
    }
    print $q->end_table, "\n";
    if ($mode eq 'create') {
        print $q->table({-width => '100%'}, $q->Tr($q->th({-class => 'sme-layout'},
                    $q->submit(-name => 'apply',
                        -value => $self->localise("APPLY")),
                    '&nbsp;',
                    $q->submit(-name => 'cancel',
                        -value => $self->localise("CANCEL")))));
    }
    elsif ($mode eq 'remove') {
        print $q->table({-width => '100%'}, $q->Tr($q->th({-class => 'sme-layout'},
                    $q->submit( -name => 'remove',
                        -value => $self->localise("REMOVE")),
                    '&nbsp;',
                    $q->submit( -name => 'cancel',
                        -value => $self->localise("CANCEL")))));
    }
    else { 
        $self->error('ERR_UNSUPPORTED_MODE');
        return undef;
    }
    $self->debug_msg("returning");

    print "</td></tr>";
    return undef;
}

=head2 remove_rule

This method is a remove wrapper for the modify method.

=cut

sub remove_rule {
    my $self = shift;
    $self->modify('remove');
}

=head2 create_new

This method is a create wrapper for the modify method.

=cut

sub create_new {
    my $self = shift;
    $self->modify('create');
}

=head2 modify

This method's purpose is to add or remove rules from the database, and then
cause the firewall rules to update.

=cut

sub modify {
    no strict 'refs';
    my $self = shift;
    my $mode = shift;
    my $q = $self->{cgi};
    $self->debug_msg("at start of modify method");

    # If the cancel button was pressed, just go back to the start page.
    if ($q->param("cancel")) {
        $self->debug_msg("the cancel button was pressed");
        $self->wherenext("Front");
    }
    else {
        # Save the changes. 
        my $proto = $q->param("protocol");
        my $sport = $q->param("source_port");
        my $dport = $q->param("destination_port");
        my $dhost = $q->param("destination_host");
        $proto =~ s/^\s+|\s+$//g;
        $sport =~ s/^\s+|\s+$//g;
        $dport =~ s/^\s+|\s+$//g;
        $dhost =~ s/^\s+|\s+$//g;

        $self->debug_msg("protocol is $proto");
        $self->debug_msg("source_port is $sport");
        $self->debug_msg("destination_port is $dport");
        $self->debug_msg("destination_host is $dhost");

        my $whichforwards = "";
        if ($proto eq 'TCP') {
            $whichforwards = 'TCPForwards';
        }
        else {
            $whichforwards = 'UDPForwards';
        }

        # Port forwarding rules are properties of the masq record under a key
        # of TCPForwards, with each one separated by commas, and the format of
        # each being, "sport,host1:dport"
        my $masq = $db->get('masq') 
            || ($self->error('ERR_NO_MASQ_RECORD') and return undef);
        $self->debug_msg("fetching $whichforwards property from masq record");
        $$whichforwards = $masq->prop($whichforwards);
        $self->debug_msg("the db property is $$whichforwards");

        if ($mode eq 'create') {
            $self->debug_msg("we are in create mode");
            my $newrule = "$sport,$dhost:$dport";
            $self->debug_msg("new rule is $newrule");
            if ($$whichforwards) {
                # Look for an identical rule. 
                my $pattern; ($pattern = $newrule) =~ s/\./\\./g;
                $self->debug_msg("looking for a dup rule; pattern $pattern");
                if ($$whichforwards =~ /$pattern/) {
                    $self->debug_msg("found a duplicate rule");
                    $self->error('ERR_DUPRULE');
                    return undef;
                }
                $$whichforwards .= ',';
            }
            else {
                $$whichforwards = '';
            }
            $$whichforwards .= $newrule;
            $self->debug_msg("\$\$whichforwards is now $$whichforwards");
        }
        elsif ($mode eq 'remove') {
            $self->debug_msg("we are in remove mode");
            if (! $$whichforwards) {
                # The category is empty. Nothing to remove.
                $self->error('ERR_CANNOT_REMOVE_NORULE');
                return undef;
            }
            my %forwards = split /,/, $$whichforwards;
            $$whichforwards = "";
            my $found = FALSE;
            foreach my $psport (keys %forwards) {
                my ($pdhost, $pdport) = split /:/, $forwards{$psport};
                $self->debug_msg("looping on $psport, $pdhost, $pdport");
                if (($sport eq $psport) &&
                    ($dport eq $pdport) &&
                    ($dhost eq $pdhost))
                {
                    $found = TRUE;
                    $self->debug_msg("found the rule to remove");
                }
                else {
                    $$whichforwards .= "$psport,$pdhost:$pdport,";
                    $self->debug_msg("\$\$whichforwards is now $$whichforwards");
                }
            }
            if (! $found) {
                $self->error('ERR_CANNOT_REMOVE_NORULE');
                return undef;
            }
            $$whichforwards =~ s/,$//;
        }

        $masq->set_prop("$whichforwards", $$whichforwards);

        system("/sbin/e-smith/signal-event",
               "remoteaccess-update") == 0
           || ($self->error('ERR_NONZERO_RETURN_EVENT') and return undef);

        #$self->wherenext("Finished");
        $self->success();
    }
    $self->debug_msg("at end of modify method");
    return undef;
}

1;
